# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from coingecko_sdk import Coingecko, AsyncCoingecko
from coingecko_sdk.types.coins.contract import (
    MarketChartGetResponse,
    MarketChartGetRangeResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestMarketChart:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip()
    @parametrize
    def test_method_get(self, client: Coingecko) -> None:
        market_chart = client.coins.contract.market_chart.get(
            contract_address="0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
            id="ethereum",
            days="days",
            vs_currency="usd",
        )
        assert_matches_type(MarketChartGetResponse, market_chart, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_get_with_all_params(self, client: Coingecko) -> None:
        market_chart = client.coins.contract.market_chart.get(
            contract_address="0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
            id="ethereum",
            days="days",
            vs_currency="usd",
            interval="5m",
            precision="full",
        )
        assert_matches_type(MarketChartGetResponse, market_chart, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_get(self, client: Coingecko) -> None:
        response = client.coins.contract.market_chart.with_raw_response.get(
            contract_address="0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
            id="ethereum",
            days="days",
            vs_currency="usd",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        market_chart = response.parse()
        assert_matches_type(MarketChartGetResponse, market_chart, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_get(self, client: Coingecko) -> None:
        with client.coins.contract.market_chart.with_streaming_response.get(
            contract_address="0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
            id="ethereum",
            days="days",
            vs_currency="usd",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            market_chart = response.parse()
            assert_matches_type(MarketChartGetResponse, market_chart, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_get(self, client: Coingecko) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.coins.contract.market_chart.with_raw_response.get(
                contract_address="0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
                id="",
                days="days",
                vs_currency="usd",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `contract_address` but received ''"):
            client.coins.contract.market_chart.with_raw_response.get(
                contract_address="",
                id="ethereum",
                days="days",
                vs_currency="usd",
            )

    @pytest.mark.skip()
    @parametrize
    def test_method_get_range(self, client: Coingecko) -> None:
        market_chart = client.coins.contract.market_chart.get_range(
            contract_address="0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
            id="ethereum",
            from_=0,
            to=0,
            vs_currency="usd",
        )
        assert_matches_type(MarketChartGetRangeResponse, market_chart, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_method_get_range_with_all_params(self, client: Coingecko) -> None:
        market_chart = client.coins.contract.market_chart.get_range(
            contract_address="0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
            id="ethereum",
            from_=0,
            to=0,
            vs_currency="usd",
            interval="5m",
            precision="full",
        )
        assert_matches_type(MarketChartGetRangeResponse, market_chart, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_raw_response_get_range(self, client: Coingecko) -> None:
        response = client.coins.contract.market_chart.with_raw_response.get_range(
            contract_address="0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
            id="ethereum",
            from_=0,
            to=0,
            vs_currency="usd",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        market_chart = response.parse()
        assert_matches_type(MarketChartGetRangeResponse, market_chart, path=["response"])

    @pytest.mark.skip()
    @parametrize
    def test_streaming_response_get_range(self, client: Coingecko) -> None:
        with client.coins.contract.market_chart.with_streaming_response.get_range(
            contract_address="0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
            id="ethereum",
            from_=0,
            to=0,
            vs_currency="usd",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            market_chart = response.parse()
            assert_matches_type(MarketChartGetRangeResponse, market_chart, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    def test_path_params_get_range(self, client: Coingecko) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.coins.contract.market_chart.with_raw_response.get_range(
                contract_address="0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
                id="",
                from_=0,
                to=0,
                vs_currency="usd",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `contract_address` but received ''"):
            client.coins.contract.market_chart.with_raw_response.get_range(
                contract_address="",
                id="ethereum",
                from_=0,
                to=0,
                vs_currency="usd",
            )


class TestAsyncMarketChart:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip()
    @parametrize
    async def test_method_get(self, async_client: AsyncCoingecko) -> None:
        market_chart = await async_client.coins.contract.market_chart.get(
            contract_address="0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
            id="ethereum",
            days="days",
            vs_currency="usd",
        )
        assert_matches_type(MarketChartGetResponse, market_chart, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_with_all_params(self, async_client: AsyncCoingecko) -> None:
        market_chart = await async_client.coins.contract.market_chart.get(
            contract_address="0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
            id="ethereum",
            days="days",
            vs_currency="usd",
            interval="5m",
            precision="full",
        )
        assert_matches_type(MarketChartGetResponse, market_chart, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_get(self, async_client: AsyncCoingecko) -> None:
        response = await async_client.coins.contract.market_chart.with_raw_response.get(
            contract_address="0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
            id="ethereum",
            days="days",
            vs_currency="usd",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        market_chart = await response.parse()
        assert_matches_type(MarketChartGetResponse, market_chart, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_get(self, async_client: AsyncCoingecko) -> None:
        async with async_client.coins.contract.market_chart.with_streaming_response.get(
            contract_address="0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
            id="ethereum",
            days="days",
            vs_currency="usd",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            market_chart = await response.parse()
            assert_matches_type(MarketChartGetResponse, market_chart, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_get(self, async_client: AsyncCoingecko) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.coins.contract.market_chart.with_raw_response.get(
                contract_address="0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
                id="",
                days="days",
                vs_currency="usd",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `contract_address` but received ''"):
            await async_client.coins.contract.market_chart.with_raw_response.get(
                contract_address="",
                id="ethereum",
                days="days",
                vs_currency="usd",
            )

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_range(self, async_client: AsyncCoingecko) -> None:
        market_chart = await async_client.coins.contract.market_chart.get_range(
            contract_address="0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
            id="ethereum",
            from_=0,
            to=0,
            vs_currency="usd",
        )
        assert_matches_type(MarketChartGetRangeResponse, market_chart, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_method_get_range_with_all_params(self, async_client: AsyncCoingecko) -> None:
        market_chart = await async_client.coins.contract.market_chart.get_range(
            contract_address="0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
            id="ethereum",
            from_=0,
            to=0,
            vs_currency="usd",
            interval="5m",
            precision="full",
        )
        assert_matches_type(MarketChartGetRangeResponse, market_chart, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_raw_response_get_range(self, async_client: AsyncCoingecko) -> None:
        response = await async_client.coins.contract.market_chart.with_raw_response.get_range(
            contract_address="0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
            id="ethereum",
            from_=0,
            to=0,
            vs_currency="usd",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        market_chart = await response.parse()
        assert_matches_type(MarketChartGetRangeResponse, market_chart, path=["response"])

    @pytest.mark.skip()
    @parametrize
    async def test_streaming_response_get_range(self, async_client: AsyncCoingecko) -> None:
        async with async_client.coins.contract.market_chart.with_streaming_response.get_range(
            contract_address="0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
            id="ethereum",
            from_=0,
            to=0,
            vs_currency="usd",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            market_chart = await response.parse()
            assert_matches_type(MarketChartGetRangeResponse, market_chart, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip()
    @parametrize
    async def test_path_params_get_range(self, async_client: AsyncCoingecko) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.coins.contract.market_chart.with_raw_response.get_range(
                contract_address="0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
                id="",
                from_=0,
                to=0,
                vs_currency="usd",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `contract_address` but received ''"):
            await async_client.coins.contract.market_chart.with_raw_response.get_range(
                contract_address="",
                id="ethereum",
                from_=0,
                to=0,
                vs_currency="usd",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.global_.decentralized_finance_defi_get_response import DecentralizedFinanceDefiGetResponse

__all__ = ["DecentralizedFinanceDefiResource", "AsyncDecentralizedFinanceDefiResource"]


class DecentralizedFinanceDefiResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DecentralizedFinanceDefiResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return DecentralizedFinanceDefiResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DecentralizedFinanceDefiResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return DecentralizedFinanceDefiResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DecentralizedFinanceDefiGetResponse:
        """
        This endpoint allows you **query top 100 cryptocurrency global decentralized
        finance (DeFi) data including DeFi market cap, trading volume**
        """
        return self._get(
            "/global/decentralized_finance_defi",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DecentralizedFinanceDefiGetResponse,
        )


class AsyncDecentralizedFinanceDefiResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDecentralizedFinanceDefiResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDecentralizedFinanceDefiResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDecentralizedFinanceDefiResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncDecentralizedFinanceDefiResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> DecentralizedFinanceDefiGetResponse:
        """
        This endpoint allows you **query top 100 cryptocurrency global decentralized
        finance (DeFi) data including DeFi market cap, trading volume**
        """
        return await self._get(
            "/global/decentralized_finance_defi",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DecentralizedFinanceDefiGetResponse,
        )


class DecentralizedFinanceDefiResourceWithRawResponse:
    def __init__(self, decentralized_finance_defi: DecentralizedFinanceDefiResource) -> None:
        self._decentralized_finance_defi = decentralized_finance_defi

        self.get = to_raw_response_wrapper(
            decentralized_finance_defi.get,
        )


class AsyncDecentralizedFinanceDefiResourceWithRawResponse:
    def __init__(self, decentralized_finance_defi: AsyncDecentralizedFinanceDefiResource) -> None:
        self._decentralized_finance_defi = decentralized_finance_defi

        self.get = async_to_raw_response_wrapper(
            decentralized_finance_defi.get,
        )


class DecentralizedFinanceDefiResourceWithStreamingResponse:
    def __init__(self, decentralized_finance_defi: DecentralizedFinanceDefiResource) -> None:
        self._decentralized_finance_defi = decentralized_finance_defi

        self.get = to_streamed_response_wrapper(
            decentralized_finance_defi.get,
        )


class AsyncDecentralizedFinanceDefiResourceWithStreamingResponse:
    def __init__(self, decentralized_finance_defi: AsyncDecentralizedFinanceDefiResource) -> None:
        self._decentralized_finance_defi = decentralized_finance_defi

        self.get = async_to_streamed_response_wrapper(
            decentralized_finance_defi.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .public_treasury_get_coin_id_response import PublicTreasuryGetCoinIDResponse as PublicTreasuryGetCoinIDResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["NFTGetListResponse"]


class NFTGetListResponse(BaseModel):
    id: Optional[str] = None
    """NFT collection ID"""

    asset_platform_id: Optional[str] = None
    """NFT collection asset platform ID"""

    contract_address: Optional[str] = None
    """NFT collection contract address"""

    name: Optional[str] = None
    """NFT collection name"""

    symbol: Optional[str] = None
    """NFT collection symbol"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["PublicTreasuryGetCoinIDResponse", "Company"]


class Company(BaseModel):
    country: Optional[str] = None
    """company incorporated country"""

    name: Optional[str] = None
    """company name"""

    percentage_of_total_supply: Optional[float] = None
    """percentage of total btc/eth supply"""

    symbol: Optional[str] = None
    """company symbol"""

    total_current_value_usd: Optional[float] = None
    """total current value of btc/eth holdings in usd"""

    total_entry_value_usd: Optional[float] = None
    """total entry value in usd"""

    total_holdings: Optional[float] = None
    """total btc/eth holdings of company"""


class PublicTreasuryGetCoinIDResponse(BaseModel):
    companies: Optional[List[Company]] = None

    market_cap_dominance: Optional[float] = None
    """market cap dominance"""

    total_holdings: Optional[float] = None
    """total btc/eth holdings of companies"""

    total_value_usd: Optional[float] = None
    """total btc/eth holdings value in usd"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...types import search_get_params
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from .trending import (
    TrendingResource,
    AsyncTrendingResource,
    TrendingResourceWithRawResponse,
    AsyncTrendingResourceWithRawResponse,
    TrendingResourceWithStreamingResponse,
    AsyncTrendingResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.search_get_response import SearchGetResponse

__all__ = ["SearchResource", "AsyncSearchResource"]


class SearchResource(SyncAPIResource):
    @cached_property
    def trending(self) -> TrendingResource:
        return TrendingResource(self._client)

    @cached_property
    def with_raw_response(self) -> SearchResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return SearchResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SearchResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return SearchResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        query: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SearchGetResponse:
        """
        This endpoint allows you to **search for coins, categories and markets listed on
        CoinGecko**

        Args:
          query: search query

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/search",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"query": query}, search_get_params.SearchGetParams),
            ),
            cast_to=SearchGetResponse,
        )


class AsyncSearchResource(AsyncAPIResource):
    @cached_property
    def trending(self) -> AsyncTrendingResource:
        return AsyncTrendingResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSearchResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSearchResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSearchResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncSearchResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        query: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SearchGetResponse:
        """
        This endpoint allows you to **search for coins, categories and markets listed on
        CoinGecko**

        Args:
          query: search query

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/search",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"query": query}, search_get_params.SearchGetParams),
            ),
            cast_to=SearchGetResponse,
        )


class SearchResourceWithRawResponse:
    def __init__(self, search: SearchResource) -> None:
        self._search = search

        self.get = to_raw_response_wrapper(
            search.get,
        )

    @cached_property
    def trending(self) -> TrendingResourceWithRawResponse:
        return TrendingResourceWithRawResponse(self._search.trending)


class AsyncSearchResourceWithRawResponse:
    def __init__(self, search: AsyncSearchResource) -> None:
        self._search = search

        self.get = async_to_raw_response_wrapper(
            search.get,
        )

    @cached_property
    def trending(self) -> AsyncTrendingResourceWithRawResponse:
        return AsyncTrendingResourceWithRawResponse(self._search.trending)


class SearchResourceWithStreamingResponse:
    def __init__(self, search: SearchResource) -> None:
        self._search = search

        self.get = to_streamed_response_wrapper(
            search.get,
        )

    @cached_property
    def trending(self) -> TrendingResourceWithStreamingResponse:
        return TrendingResourceWithStreamingResponse(self._search.trending)


class AsyncSearchResourceWithStreamingResponse:
    def __init__(self, search: AsyncSearchResource) -> None:
        self._search = search

        self.get = async_to_streamed_response_wrapper(
            search.get,
        )

    @cached_property
    def trending(self) -> AsyncTrendingResourceWithStreamingResponse:
        return AsyncTrendingResourceWithStreamingResponse(self._search.trending)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["TickerGetResponse", "Ticker", "TickerConvertedLast", "TickerConvertedVolume", "TickerMarket"]


class TickerConvertedLast(BaseModel):
    btc: Optional[float] = None

    eth: Optional[float] = None

    usd: Optional[float] = None


class TickerConvertedVolume(BaseModel):
    btc: Optional[float] = None

    eth: Optional[float] = None

    usd: Optional[float] = None


class TickerMarket(BaseModel):
    has_trading_incentive: bool
    """exchange trading incentive"""

    identifier: str
    """exchange identifier"""

    name: str
    """exchange name"""

    logo: Optional[str] = None
    """exchange image url"""


class Ticker(BaseModel):
    base: Optional[str] = None
    """coin ticker base currency"""

    bid_ask_spread_percentage: Optional[float] = None
    """coin ticker bid ask spread percentage"""

    coin_id: Optional[str] = None
    """coin ticker base currency coin ID"""

    converted_last: Optional[TickerConvertedLast] = None
    """coin ticker converted last price"""

    converted_volume: Optional[TickerConvertedVolume] = None
    """coin ticker converted volume"""

    cost_to_move_down_usd: Optional[float] = None
    """coin ticker cost to move down in usd"""

    cost_to_move_up_usd: Optional[float] = None
    """coin ticker cost to move up in usd"""

    is_anomaly: Optional[bool] = None
    """coin ticker anomaly"""

    is_stale: Optional[bool] = None
    """coin ticker stale"""

    last: Optional[float] = None
    """coin ticker last price"""

    last_fetch_at: Optional[str] = None
    """coin ticker last fetch timestamp"""

    last_traded_at: Optional[str] = None
    """coin ticker last traded timestamp"""

    market: Optional[TickerMarket] = None
    """coin ticker exchange"""

    target: Optional[str] = None
    """coin ticker target currency"""

    target_coin_id: Optional[str] = None
    """coin ticker target currency coin ID"""

    timestamp: Optional[str] = None
    """coin ticker timestamp"""

    token_info_url: Optional[str] = None
    """coin ticker token info url"""

    trade_url: Optional[str] = None
    """coin ticker trade url"""

    trust_score: Optional[str] = None
    """coin ticker trust score"""

    volume: Optional[float] = None
    """coin ticker volume"""


class TickerGetResponse(BaseModel):
    name: Optional[str] = None
    """coin name"""

    tickers: Optional[List[Ticker]] = None
    """list of tickers"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ..._compat import cached_property
from .categories import (
    CategoriesResource,
    AsyncCategoriesResource,
    CategoriesResourceWithRawResponse,
    AsyncCategoriesResourceWithRawResponse,
    CategoriesResourceWithStreamingResponse,
    AsyncCategoriesResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from .pools.pools import (
    PoolsResource,
    AsyncPoolsResource,
    PoolsResourceWithRawResponse,
    AsyncPoolsResourceWithRawResponse,
    PoolsResourceWithStreamingResponse,
    AsyncPoolsResourceWithStreamingResponse,
)
from .search.search import (
    SearchResource,
    AsyncSearchResource,
    SearchResourceWithRawResponse,
    AsyncSearchResourceWithRawResponse,
    SearchResourceWithStreamingResponse,
    AsyncSearchResourceWithStreamingResponse,
)
from .simple.simple import (
    SimpleResource,
    AsyncSimpleResource,
    SimpleResourceWithRawResponse,
    AsyncSimpleResourceWithRawResponse,
    SimpleResourceWithStreamingResponse,
    AsyncSimpleResourceWithStreamingResponse,
)
from .tokens.tokens import (
    TokensResource,
    AsyncTokensResource,
    TokensResourceWithRawResponse,
    AsyncTokensResourceWithRawResponse,
    TokensResourceWithStreamingResponse,
    AsyncTokensResourceWithStreamingResponse,
)
from .networks.networks import (
    NetworksResource,
    AsyncNetworksResource,
    NetworksResourceWithRawResponse,
    AsyncNetworksResourceWithRawResponse,
    NetworksResourceWithStreamingResponse,
    AsyncNetworksResourceWithStreamingResponse,
)

__all__ = ["OnchainResource", "AsyncOnchainResource"]


class OnchainResource(SyncAPIResource):
    @cached_property
    def categories(self) -> CategoriesResource:
        return CategoriesResource(self._client)

    @cached_property
    def networks(self) -> NetworksResource:
        return NetworksResource(self._client)

    @cached_property
    def pools(self) -> PoolsResource:
        return PoolsResource(self._client)

    @cached_property
    def search(self) -> SearchResource:
        return SearchResource(self._client)

    @cached_property
    def simple(self) -> SimpleResource:
        return SimpleResource(self._client)

    @cached_property
    def tokens(self) -> TokensResource:
        return TokensResource(self._client)

    @cached_property
    def with_raw_response(self) -> OnchainResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return OnchainResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OnchainResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return OnchainResourceWithStreamingResponse(self)


class AsyncOnchainResource(AsyncAPIResource):
    @cached_property
    def categories(self) -> AsyncCategoriesResource:
        return AsyncCategoriesResource(self._client)

    @cached_property
    def networks(self) -> AsyncNetworksResource:
        return AsyncNetworksResource(self._client)

    @cached_property
    def pools(self) -> AsyncPoolsResource:
        return AsyncPoolsResource(self._client)

    @cached_property
    def search(self) -> AsyncSearchResource:
        return AsyncSearchResource(self._client)

    @cached_property
    def simple(self) -> AsyncSimpleResource:
        return AsyncSimpleResource(self._client)

    @cached_property
    def tokens(self) -> AsyncTokensResource:
        return AsyncTokensResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncOnchainResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOnchainResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOnchainResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncOnchainResourceWithStreamingResponse(self)


class OnchainResourceWithRawResponse:
    def __init__(self, onchain: OnchainResource) -> None:
        self._onchain = onchain

    @cached_property
    def categories(self) -> CategoriesResourceWithRawResponse:
        return CategoriesResourceWithRawResponse(self._onchain.categories)

    @cached_property
    def networks(self) -> NetworksResourceWithRawResponse:
        return NetworksResourceWithRawResponse(self._onchain.networks)

    @cached_property
    def pools(self) -> PoolsResourceWithRawResponse:
        return PoolsResourceWithRawResponse(self._onchain.pools)

    @cached_property
    def search(self) -> SearchResourceWithRawResponse:
        return SearchResourceWithRawResponse(self._onchain.search)

    @cached_property
    def simple(self) -> SimpleResourceWithRawResponse:
        return SimpleResourceWithRawResponse(self._onchain.simple)

    @cached_property
    def tokens(self) -> TokensResourceWithRawResponse:
        return TokensResourceWithRawResponse(self._onchain.tokens)


class AsyncOnchainResourceWithRawResponse:
    def __init__(self, onchain: AsyncOnchainResource) -> None:
        self._onchain = onchain

    @cached_property
    def categories(self) -> AsyncCategoriesResourceWithRawResponse:
        return AsyncCategoriesResourceWithRawResponse(self._onchain.categories)

    @cached_property
    def networks(self) -> AsyncNetworksResourceWithRawResponse:
        return AsyncNetworksResourceWithRawResponse(self._onchain.networks)

    @cached_property
    def pools(self) -> AsyncPoolsResourceWithRawResponse:
        return AsyncPoolsResourceWithRawResponse(self._onchain.pools)

    @cached_property
    def search(self) -> AsyncSearchResourceWithRawResponse:
        return AsyncSearchResourceWithRawResponse(self._onchain.search)

    @cached_property
    def simple(self) -> AsyncSimpleResourceWithRawResponse:
        return AsyncSimpleResourceWithRawResponse(self._onchain.simple)

    @cached_property
    def tokens(self) -> AsyncTokensResourceWithRawResponse:
        return AsyncTokensResourceWithRawResponse(self._onchain.tokens)


class OnchainResourceWithStreamingResponse:
    def __init__(self, onchain: OnchainResource) -> None:
        self._onchain = onchain

    @cached_property
    def categories(self) -> CategoriesResourceWithStreamingResponse:
        return CategoriesResourceWithStreamingResponse(self._onchain.categories)

    @cached_property
    def networks(self) -> NetworksResourceWithStreamingResponse:
        return NetworksResourceWithStreamingResponse(self._onchain.networks)

    @cached_property
    def pools(self) -> PoolsResourceWithStreamingResponse:
        return PoolsResourceWithStreamingResponse(self._onchain.pools)

    @cached_property
    def search(self) -> SearchResourceWithStreamingResponse:
        return SearchResourceWithStreamingResponse(self._onchain.search)

    @cached_property
    def simple(self) -> SimpleResourceWithStreamingResponse:
        return SimpleResourceWithStreamingResponse(self._onchain.simple)

    @cached_property
    def tokens(self) -> TokensResourceWithStreamingResponse:
        return TokensResourceWithStreamingResponse(self._onchain.tokens)


class AsyncOnchainResourceWithStreamingResponse:
    def __init__(self, onchain: AsyncOnchainResource) -> None:
        self._onchain = onchain

    @cached_property
    def categories(self) -> AsyncCategoriesResourceWithStreamingResponse:
        return AsyncCategoriesResourceWithStreamingResponse(self._onchain.categories)

    @cached_property
    def networks(self) -> AsyncNetworksResourceWithStreamingResponse:
        return AsyncNetworksResourceWithStreamingResponse(self._onchain.networks)

    @cached_property
    def pools(self) -> AsyncPoolsResourceWithStreamingResponse:
        return AsyncPoolsResourceWithStreamingResponse(self._onchain.pools)

    @cached_property
    def search(self) -> AsyncSearchResourceWithStreamingResponse:
        return AsyncSearchResourceWithStreamingResponse(self._onchain.search)

    @cached_property
    def simple(self) -> AsyncSimpleResourceWithStreamingResponse:
        return AsyncSimpleResourceWithStreamingResponse(self._onchain.simple)

    @cached_property
    def tokens(self) -> AsyncTokensResourceWithStreamingResponse:
        return AsyncTokensResourceWithStreamingResponse(self._onchain.tokens)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["TotalSupplyChartGetRangeParams"]


class TotalSupplyChartGetRangeParams(TypedDict, total=False):
    from_: Required[Annotated[float, PropertyInfo(alias="from")]]
    """starting date in UNIX timestamp"""

    to: Required[float]
    """ending date in UNIX timestamp"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .exchange_get_params import ExchangeGetParams as ExchangeGetParams
from .exchange_get_response import ExchangeGetResponse as ExchangeGetResponse
from .exchange_get_id_params import ExchangeGetIDParams as ExchangeGetIDParams
from .exchange_get_id_response import ExchangeGetIDResponse as ExchangeGetIDResponse
from .exchange_get_list_response import ExchangeGetListResponse as ExchangeGetListResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["TickerGetResponse", "Ticker"]


class Ticker(BaseModel):
    floor_price_in_native_currency: Optional[float] = None
    """NFT collection floor price in native currency"""

    h24_volume_in_native_currency: Optional[float] = None
    """NFT collection volume in 24 hours in native currency"""

    image: Optional[str] = None
    """NFT marketplace image url"""

    name: Optional[str] = None
    """NFT marketplace name"""

    native_currency: Optional[str] = None
    """NFT collection native currency"""

    native_currency_symbol: Optional[str] = None
    """NFT collection native currency symbol"""

    nft_collection_url: Optional[str] = None
    """NFT collection url in the NFT marketplace"""

    nft_marketplace_id: Optional[str] = None
    """NFT marketplace ID"""

    updated_at: Optional[str] = None
    """last updated time"""


class TickerGetResponse(BaseModel):
    tickers: Optional[List[Ticker]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["CategoryGetParams"]


class CategoryGetParams(TypedDict, total=False):
    page: int
    """page through results Default value: `1`"""

    sort: Literal[
        "h1_volume_percentage_desc",
        "h6_volume_percentage_desc",
        "h12_volume_percentage_desc",
        "h24_tx_count_desc",
        "h24_volume_usd_desc",
        "fdv_usd_desc",
        "reserve_in_usd_desc",
    ]
    """sort the categories by field Default value: `h6_volume_percentage_desc`"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...types import exchange_get_params, exchange_get_id_params, exchange_get_list_params
from .tickers import (
    TickersResource,
    AsyncTickersResource,
    TickersResourceWithRawResponse,
    AsyncTickersResourceWithRawResponse,
    TickersResourceWithStreamingResponse,
    AsyncTickersResourceWithStreamingResponse,
)
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .volume_chart import (
    VolumeChartResource,
    AsyncVolumeChartResource,
    VolumeChartResourceWithRawResponse,
    AsyncVolumeChartResourceWithRawResponse,
    VolumeChartResourceWithStreamingResponse,
    AsyncVolumeChartResourceWithStreamingResponse,
)
from ..._base_client import make_request_options
from ...types.exchange_get_response import ExchangeGetResponse
from ...types.exchange_get_id_response import ExchangeGetIDResponse
from ...types.exchange_get_list_response import ExchangeGetListResponse

__all__ = ["ExchangesResource", "AsyncExchangesResource"]


class ExchangesResource(SyncAPIResource):
    @cached_property
    def tickers(self) -> TickersResource:
        return TickersResource(self._client)

    @cached_property
    def volume_chart(self) -> VolumeChartResource:
        return VolumeChartResource(self._client)

    @cached_property
    def with_raw_response(self) -> ExchangesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return ExchangesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ExchangesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return ExchangesResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        page: float | NotGiven = NOT_GIVEN,
        per_page: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExchangeGetResponse:
        """
        This endpoint allows you to **query all the supported exchanges with exchanges’
        data (ID, name, country, ...) that have active trading volumes on CoinGecko**

        Args:
          page: page through results, default: 1

          per_page: total results per page, default: 100 Valid values: 1...250

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/exchanges",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    exchange_get_params.ExchangeGetParams,
                ),
            ),
            cast_to=ExchangeGetResponse,
        )

    def get_id(
        self,
        id: str,
        *,
        dex_pair_format: Literal["contract_address", "symbol"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExchangeGetIDResponse:
        """
        This endpoint allows you to **query exchange’s data (name, year established,
        country, ...), exchange volume in BTC and top 100 tickers based on exchange’s
        ID**

        Args:
          dex_pair_format:
              set to `symbol` to display DEX pair base and target as symbols, default:
              `contract_address`

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/exchanges/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"dex_pair_format": dex_pair_format}, exchange_get_id_params.ExchangeGetIDParams),
            ),
            cast_to=ExchangeGetIDResponse,
        )

    def get_list(
        self,
        *,
        status: Literal["active", "inactive"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExchangeGetListResponse:
        """
        This endpoint allows you to **query all the exchanges with ID and name**

        Args:
          status: filter by status of exchanges, default: active

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/exchanges/list",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"status": status}, exchange_get_list_params.ExchangeGetListParams),
            ),
            cast_to=ExchangeGetListResponse,
        )


class AsyncExchangesResource(AsyncAPIResource):
    @cached_property
    def tickers(self) -> AsyncTickersResource:
        return AsyncTickersResource(self._client)

    @cached_property
    def volume_chart(self) -> AsyncVolumeChartResource:
        return AsyncVolumeChartResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncExchangesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncExchangesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncExchangesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncExchangesResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        page: float | NotGiven = NOT_GIVEN,
        per_page: float | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExchangeGetResponse:
        """
        This endpoint allows you to **query all the supported exchanges with exchanges’
        data (ID, name, country, ...) that have active trading volumes on CoinGecko**

        Args:
          page: page through results, default: 1

          per_page: total results per page, default: 100 Valid values: 1...250

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/exchanges",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    exchange_get_params.ExchangeGetParams,
                ),
            ),
            cast_to=ExchangeGetResponse,
        )

    async def get_id(
        self,
        id: str,
        *,
        dex_pair_format: Literal["contract_address", "symbol"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExchangeGetIDResponse:
        """
        This endpoint allows you to **query exchange’s data (name, year established,
        country, ...), exchange volume in BTC and top 100 tickers based on exchange’s
        ID**

        Args:
          dex_pair_format:
              set to `symbol` to display DEX pair base and target as symbols, default:
              `contract_address`

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/exchanges/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"dex_pair_format": dex_pair_format}, exchange_get_id_params.ExchangeGetIDParams
                ),
            ),
            cast_to=ExchangeGetIDResponse,
        )

    async def get_list(
        self,
        *,
        status: Literal["active", "inactive"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ExchangeGetListResponse:
        """
        This endpoint allows you to **query all the exchanges with ID and name**

        Args:
          status: filter by status of exchanges, default: active

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/exchanges/list",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"status": status}, exchange_get_list_params.ExchangeGetListParams),
            ),
            cast_to=ExchangeGetListResponse,
        )


class ExchangesResourceWithRawResponse:
    def __init__(self, exchanges: ExchangesResource) -> None:
        self._exchanges = exchanges

        self.get = to_raw_response_wrapper(
            exchanges.get,
        )
        self.get_id = to_raw_response_wrapper(
            exchanges.get_id,
        )
        self.get_list = to_raw_response_wrapper(
            exchanges.get_list,
        )

    @cached_property
    def tickers(self) -> TickersResourceWithRawResponse:
        return TickersResourceWithRawResponse(self._exchanges.tickers)

    @cached_property
    def volume_chart(self) -> VolumeChartResourceWithRawResponse:
        return VolumeChartResourceWithRawResponse(self._exchanges.volume_chart)


class AsyncExchangesResourceWithRawResponse:
    def __init__(self, exchanges: AsyncExchangesResource) -> None:
        self._exchanges = exchanges

        self.get = async_to_raw_response_wrapper(
            exchanges.get,
        )
        self.get_id = async_to_raw_response_wrapper(
            exchanges.get_id,
        )
        self.get_list = async_to_raw_response_wrapper(
            exchanges.get_list,
        )

    @cached_property
    def tickers(self) -> AsyncTickersResourceWithRawResponse:
        return AsyncTickersResourceWithRawResponse(self._exchanges.tickers)

    @cached_property
    def volume_chart(self) -> AsyncVolumeChartResourceWithRawResponse:
        return AsyncVolumeChartResourceWithRawResponse(self._exchanges.volume_chart)


class ExchangesResourceWithStreamingResponse:
    def __init__(self, exchanges: ExchangesResource) -> None:
        self._exchanges = exchanges

        self.get = to_streamed_response_wrapper(
            exchanges.get,
        )
        self.get_id = to_streamed_response_wrapper(
            exchanges.get_id,
        )
        self.get_list = to_streamed_response_wrapper(
            exchanges.get_list,
        )

    @cached_property
    def tickers(self) -> TickersResourceWithStreamingResponse:
        return TickersResourceWithStreamingResponse(self._exchanges.tickers)

    @cached_property
    def volume_chart(self) -> VolumeChartResourceWithStreamingResponse:
        return VolumeChartResourceWithStreamingResponse(self._exchanges.volume_chart)


class AsyncExchangesResourceWithStreamingResponse:
    def __init__(self, exchanges: AsyncExchangesResource) -> None:
        self._exchanges = exchanges

        self.get = async_to_streamed_response_wrapper(
            exchanges.get,
        )
        self.get_id = async_to_streamed_response_wrapper(
            exchanges.get_id,
        )
        self.get_list = async_to_streamed_response_wrapper(
            exchanges.get_list,
        )

    @cached_property
    def tickers(self) -> AsyncTickersResourceWithStreamingResponse:
        return AsyncTickersResourceWithStreamingResponse(self._exchanges.tickers)

    @cached_property
    def volume_chart(self) -> AsyncVolumeChartResourceWithStreamingResponse:
        return AsyncVolumeChartResourceWithStreamingResponse(self._exchanges.volume_chart)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.onchain import category_get_params, category_get_pools_params
from ...types.onchain.category_get_response import CategoryGetResponse
from ...types.onchain.category_get_pools_response import CategoryGetPoolsResponse

__all__ = ["CategoriesResource", "AsyncCategoriesResource"]


class CategoriesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CategoriesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return CategoriesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CategoriesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return CategoriesResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        page: int | NotGiven = NOT_GIVEN,
        sort: Literal[
            "h1_volume_percentage_desc",
            "h6_volume_percentage_desc",
            "h12_volume_percentage_desc",
            "h24_tx_count_desc",
            "h24_volume_usd_desc",
            "fdv_usd_desc",
            "reserve_in_usd_desc",
        ]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CategoryGetResponse:
        """
        This endpoint allows you to **query all the supported categories on
        GeckoTerminal**

        Args:
          page: page through results Default value: `1`

          sort: sort the categories by field Default value: `h6_volume_percentage_desc`

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/onchain/categories",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "sort": sort,
                    },
                    category_get_params.CategoryGetParams,
                ),
            ),
            cast_to=CategoryGetResponse,
        )

    def get_pools(
        self,
        category_id: str,
        *,
        include: str | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        sort: Literal[
            "m5_trending",
            "h1_trending",
            "h6_trending",
            "h24_trending",
            "h24_tx_count_desc",
            "h24_volume_usd_desc",
            "pool_created_at_desc",
            "h24_price_change_percentage_desc",
        ]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CategoryGetPoolsResponse:
        """
        This endpoint allows you to **query all the pools based on the provided category
        ID**

        Args:
          include: attributes to include, comma-separated if more than one to include Available
              values: `base_token`, `quote_token`, `dex`, `network`. Example: `base_token` or
              `base_token,dex`

          page: page through results Default value: `1`

          sort: sort the pools by field Default value: `pool_created_at_desc`

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not category_id:
            raise ValueError(f"Expected a non-empty value for `category_id` but received {category_id!r}")
        return self._get(
            f"/onchain/categories/{category_id}/pools",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include": include,
                        "page": page,
                        "sort": sort,
                    },
                    category_get_pools_params.CategoryGetPoolsParams,
                ),
            ),
            cast_to=CategoryGetPoolsResponse,
        )


class AsyncCategoriesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCategoriesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCategoriesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCategoriesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncCategoriesResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        page: int | NotGiven = NOT_GIVEN,
        sort: Literal[
            "h1_volume_percentage_desc",
            "h6_volume_percentage_desc",
            "h12_volume_percentage_desc",
            "h24_tx_count_desc",
            "h24_volume_usd_desc",
            "fdv_usd_desc",
            "reserve_in_usd_desc",
        ]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CategoryGetResponse:
        """
        This endpoint allows you to **query all the supported categories on
        GeckoTerminal**

        Args:
          page: page through results Default value: `1`

          sort: sort the categories by field Default value: `h6_volume_percentage_desc`

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/onchain/categories",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "page": page,
                        "sort": sort,
                    },
                    category_get_params.CategoryGetParams,
                ),
            ),
            cast_to=CategoryGetResponse,
        )

    async def get_pools(
        self,
        category_id: str,
        *,
        include: str | NotGiven = NOT_GIVEN,
        page: int | NotGiven = NOT_GIVEN,
        sort: Literal[
            "m5_trending",
            "h1_trending",
            "h6_trending",
            "h24_trending",
            "h24_tx_count_desc",
            "h24_volume_usd_desc",
            "pool_created_at_desc",
            "h24_price_change_percentage_desc",
        ]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CategoryGetPoolsResponse:
        """
        This endpoint allows you to **query all the pools based on the provided category
        ID**

        Args:
          include: attributes to include, comma-separated if more than one to include Available
              values: `base_token`, `quote_token`, `dex`, `network`. Example: `base_token` or
              `base_token,dex`

          page: page through results Default value: `1`

          sort: sort the pools by field Default value: `pool_created_at_desc`

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not category_id:
            raise ValueError(f"Expected a non-empty value for `category_id` but received {category_id!r}")
        return await self._get(
            f"/onchain/categories/{category_id}/pools",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "include": include,
                        "page": page,
                        "sort": sort,
                    },
                    category_get_pools_params.CategoryGetPoolsParams,
                ),
            ),
            cast_to=CategoryGetPoolsResponse,
        )


class CategoriesResourceWithRawResponse:
    def __init__(self, categories: CategoriesResource) -> None:
        self._categories = categories

        self.get = to_raw_response_wrapper(
            categories.get,
        )
        self.get_pools = to_raw_response_wrapper(
            categories.get_pools,
        )


class AsyncCategoriesResourceWithRawResponse:
    def __init__(self, categories: AsyncCategoriesResource) -> None:
        self._categories = categories

        self.get = async_to_raw_response_wrapper(
            categories.get,
        )
        self.get_pools = async_to_raw_response_wrapper(
            categories.get_pools,
        )


class CategoriesResourceWithStreamingResponse:
    def __init__(self, categories: CategoriesResource) -> None:
        self._categories = categories

        self.get = to_streamed_response_wrapper(
            categories.get,
        )
        self.get_pools = to_streamed_response_wrapper(
            categories.get_pools,
        )


class AsyncCategoriesResourceWithStreamingResponse:
    def __init__(self, categories: AsyncCategoriesResource) -> None:
        self._categories = categories

        self.get = async_to_streamed_response_wrapper(
            categories.get,
        )
        self.get_pools = async_to_streamed_response_wrapper(
            categories.get_pools,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .price import (
    PriceResource,
    AsyncPriceResource,
    PriceResourceWithRawResponse,
    AsyncPriceResourceWithRawResponse,
    PriceResourceWithStreamingResponse,
    AsyncPriceResourceWithStreamingResponse,
)
from .simple import (
    SimpleResource,
    AsyncSimpleResource,
    SimpleResourceWithRawResponse,
    AsyncSimpleResourceWithRawResponse,
    SimpleResourceWithStreamingResponse,
    AsyncSimpleResourceWithStreamingResponse,
)
from .token_price import (
    TokenPriceResource,
    AsyncTokenPriceResource,
    TokenPriceResourceWithRawResponse,
    AsyncTokenPriceResourceWithRawResponse,
    TokenPriceResourceWithStreamingResponse,
    AsyncTokenPriceResourceWithStreamingResponse,
)
from .supported_vs_currencies import (
    SupportedVsCurrenciesResource,
    AsyncSupportedVsCurrenciesResource,
    SupportedVsCurrenciesResourceWithRawResponse,
    AsyncSupportedVsCurrenciesResourceWithRawResponse,
    SupportedVsCurrenciesResourceWithStreamingResponse,
    AsyncSupportedVsCurrenciesResourceWithStreamingResponse,
)

__all__ = [
    "PriceResource",
    "AsyncPriceResource",
    "PriceResourceWithRawResponse",
    "AsyncPriceResourceWithRawResponse",
    "PriceResourceWithStreamingResponse",
    "AsyncPriceResourceWithStreamingResponse",
    "SupportedVsCurrenciesResource",
    "AsyncSupportedVsCurrenciesResource",
    "SupportedVsCurrenciesResourceWithRawResponse",
    "AsyncSupportedVsCurrenciesResourceWithRawResponse",
    "SupportedVsCurrenciesResourceWithStreamingResponse",
    "AsyncSupportedVsCurrenciesResourceWithStreamingResponse",
    "TokenPriceResource",
    "AsyncTokenPriceResource",
    "TokenPriceResourceWithRawResponse",
    "AsyncTokenPriceResourceWithRawResponse",
    "TokenPriceResourceWithStreamingResponse",
    "AsyncTokenPriceResourceWithStreamingResponse",
    "SimpleResource",
    "AsyncSimpleResource",
    "SimpleResourceWithRawResponse",
    "AsyncSimpleResourceWithRawResponse",
    "SimpleResourceWithStreamingResponse",
    "AsyncSimpleResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from .list import (
    ListResource,
    AsyncListResource,
    ListResourceWithRawResponse,
    AsyncListResourceWithRawResponse,
    ListResourceWithStreamingResponse,
    AsyncListResourceWithStreamingResponse,
)
from .ohlc import (
    OhlcResource,
    AsyncOhlcResource,
    OhlcResourceWithRawResponse,
    AsyncOhlcResourceWithRawResponse,
    OhlcResourceWithStreamingResponse,
    AsyncOhlcResourceWithStreamingResponse,
)
from ...types import coin_get_id_params
from .history import (
    HistoryResource,
    AsyncHistoryResource,
    HistoryResourceWithRawResponse,
    AsyncHistoryResourceWithRawResponse,
    HistoryResourceWithStreamingResponse,
    AsyncHistoryResourceWithStreamingResponse,
)
from .markets import (
    MarketsResource,
    AsyncMarketsResource,
    MarketsResourceWithRawResponse,
    AsyncMarketsResourceWithRawResponse,
    MarketsResourceWithStreamingResponse,
    AsyncMarketsResourceWithStreamingResponse,
)
from .tickers import (
    TickersResource,
    AsyncTickersResource,
    TickersResourceWithRawResponse,
    AsyncTickersResourceWithRawResponse,
    TickersResourceWithStreamingResponse,
    AsyncTickersResourceWithStreamingResponse,
)
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from .categories import (
    CategoriesResource,
    AsyncCategoriesResource,
    CategoriesResourceWithRawResponse,
    AsyncCategoriesResourceWithRawResponse,
    CategoriesResourceWithStreamingResponse,
    AsyncCategoriesResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .market_chart import (
    MarketChartResource,
    AsyncMarketChartResource,
    MarketChartResourceWithRawResponse,
    AsyncMarketChartResourceWithRawResponse,
    MarketChartResourceWithStreamingResponse,
    AsyncMarketChartResourceWithStreamingResponse,
)
from ..._base_client import make_request_options
from .contract.contract import (
    ContractResource,
    AsyncContractResource,
    ContractResourceWithRawResponse,
    AsyncContractResourceWithRawResponse,
    ContractResourceWithStreamingResponse,
    AsyncContractResourceWithStreamingResponse,
)
from .top_gainers_losers import (
    TopGainersLosersResource,
    AsyncTopGainersLosersResource,
    TopGainersLosersResourceWithRawResponse,
    AsyncTopGainersLosersResourceWithRawResponse,
    TopGainersLosersResourceWithStreamingResponse,
    AsyncTopGainersLosersResourceWithStreamingResponse,
)
from .total_supply_chart import (
    TotalSupplyChartResource,
    AsyncTotalSupplyChartResource,
    TotalSupplyChartResourceWithRawResponse,
    AsyncTotalSupplyChartResourceWithRawResponse,
    TotalSupplyChartResourceWithStreamingResponse,
    AsyncTotalSupplyChartResourceWithStreamingResponse,
)
from .circulating_supply_chart import (
    CirculatingSupplyChartResource,
    AsyncCirculatingSupplyChartResource,
    CirculatingSupplyChartResourceWithRawResponse,
    AsyncCirculatingSupplyChartResourceWithRawResponse,
    CirculatingSupplyChartResourceWithStreamingResponse,
    AsyncCirculatingSupplyChartResourceWithStreamingResponse,
)
from ...types.coin_get_id_response import CoinGetIDResponse

__all__ = ["CoinsResource", "AsyncCoinsResource"]


class CoinsResource(SyncAPIResource):
    @cached_property
    def categories(self) -> CategoriesResource:
        return CategoriesResource(self._client)

    @cached_property
    def list(self) -> ListResource:
        return ListResource(self._client)

    @cached_property
    def markets(self) -> MarketsResource:
        return MarketsResource(self._client)

    @cached_property
    def top_gainers_losers(self) -> TopGainersLosersResource:
        return TopGainersLosersResource(self._client)

    @cached_property
    def circulating_supply_chart(self) -> CirculatingSupplyChartResource:
        return CirculatingSupplyChartResource(self._client)

    @cached_property
    def contract(self) -> ContractResource:
        return ContractResource(self._client)

    @cached_property
    def history(self) -> HistoryResource:
        return HistoryResource(self._client)

    @cached_property
    def market_chart(self) -> MarketChartResource:
        return MarketChartResource(self._client)

    @cached_property
    def ohlc(self) -> OhlcResource:
        return OhlcResource(self._client)

    @cached_property
    def tickers(self) -> TickersResource:
        return TickersResource(self._client)

    @cached_property
    def total_supply_chart(self) -> TotalSupplyChartResource:
        return TotalSupplyChartResource(self._client)

    @cached_property
    def with_raw_response(self) -> CoinsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return CoinsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CoinsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return CoinsResourceWithStreamingResponse(self)

    def get_id(
        self,
        id: str,
        *,
        community_data: bool | NotGiven = NOT_GIVEN,
        developer_data: bool | NotGiven = NOT_GIVEN,
        dex_pair_format: Literal["contract_address", "symbol"] | NotGiven = NOT_GIVEN,
        localization: bool | NotGiven = NOT_GIVEN,
        market_data: bool | NotGiven = NOT_GIVEN,
        sparkline: bool | NotGiven = NOT_GIVEN,
        tickers: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CoinGetIDResponse:
        """
        This endpoint allows you to **query all the metadata (image, websites, socials,
        description, contract address, etc.) and market data (price, ATH, exchange
        tickers, etc.) of a coin from the CoinGecko coin page based on a particular coin
        ID**

        Args:
          community_data: include community data, default: true

          developer_data: include developer data, default: true

          dex_pair_format:
              set to `symbol` to display DEX pair base and target as symbols, default:
              `contract_address`

          localization: include all the localized languages in the response, default: true

          market_data: include market data, default: true

          sparkline: include sparkline 7 days data, default: false

          tickers: include tickers data, default: true

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/coins/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "community_data": community_data,
                        "developer_data": developer_data,
                        "dex_pair_format": dex_pair_format,
                        "localization": localization,
                        "market_data": market_data,
                        "sparkline": sparkline,
                        "tickers": tickers,
                    },
                    coin_get_id_params.CoinGetIDParams,
                ),
            ),
            cast_to=CoinGetIDResponse,
        )


class AsyncCoinsResource(AsyncAPIResource):
    @cached_property
    def categories(self) -> AsyncCategoriesResource:
        return AsyncCategoriesResource(self._client)

    @cached_property
    def list(self) -> AsyncListResource:
        return AsyncListResource(self._client)

    @cached_property
    def markets(self) -> AsyncMarketsResource:
        return AsyncMarketsResource(self._client)

    @cached_property
    def top_gainers_losers(self) -> AsyncTopGainersLosersResource:
        return AsyncTopGainersLosersResource(self._client)

    @cached_property
    def circulating_supply_chart(self) -> AsyncCirculatingSupplyChartResource:
        return AsyncCirculatingSupplyChartResource(self._client)

    @cached_property
    def contract(self) -> AsyncContractResource:
        return AsyncContractResource(self._client)

    @cached_property
    def history(self) -> AsyncHistoryResource:
        return AsyncHistoryResource(self._client)

    @cached_property
    def market_chart(self) -> AsyncMarketChartResource:
        return AsyncMarketChartResource(self._client)

    @cached_property
    def ohlc(self) -> AsyncOhlcResource:
        return AsyncOhlcResource(self._client)

    @cached_property
    def tickers(self) -> AsyncTickersResource:
        return AsyncTickersResource(self._client)

    @cached_property
    def total_supply_chart(self) -> AsyncTotalSupplyChartResource:
        return AsyncTotalSupplyChartResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncCoinsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCoinsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCoinsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncCoinsResourceWithStreamingResponse(self)

    async def get_id(
        self,
        id: str,
        *,
        community_data: bool | NotGiven = NOT_GIVEN,
        developer_data: bool | NotGiven = NOT_GIVEN,
        dex_pair_format: Literal["contract_address", "symbol"] | NotGiven = NOT_GIVEN,
        localization: bool | NotGiven = NOT_GIVEN,
        market_data: bool | NotGiven = NOT_GIVEN,
        sparkline: bool | NotGiven = NOT_GIVEN,
        tickers: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> CoinGetIDResponse:
        """
        This endpoint allows you to **query all the metadata (image, websites, socials,
        description, contract address, etc.) and market data (price, ATH, exchange
        tickers, etc.) of a coin from the CoinGecko coin page based on a particular coin
        ID**

        Args:
          community_data: include community data, default: true

          developer_data: include developer data, default: true

          dex_pair_format:
              set to `symbol` to display DEX pair base and target as symbols, default:
              `contract_address`

          localization: include all the localized languages in the response, default: true

          market_data: include market data, default: true

          sparkline: include sparkline 7 days data, default: false

          tickers: include tickers data, default: true

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/coins/{id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "community_data": community_data,
                        "developer_data": developer_data,
                        "dex_pair_format": dex_pair_format,
                        "localization": localization,
                        "market_data": market_data,
                        "sparkline": sparkline,
                        "tickers": tickers,
                    },
                    coin_get_id_params.CoinGetIDParams,
                ),
            ),
            cast_to=CoinGetIDResponse,
        )


class CoinsResourceWithRawResponse:
    def __init__(self, coins: CoinsResource) -> None:
        self._coins = coins

        self.get_id = to_raw_response_wrapper(
            coins.get_id,
        )

    @cached_property
    def categories(self) -> CategoriesResourceWithRawResponse:
        return CategoriesResourceWithRawResponse(self._coins.categories)

    @cached_property
    def list(self) -> ListResourceWithRawResponse:
        return ListResourceWithRawResponse(self._coins.list)

    @cached_property
    def markets(self) -> MarketsResourceWithRawResponse:
        return MarketsResourceWithRawResponse(self._coins.markets)

    @cached_property
    def top_gainers_losers(self) -> TopGainersLosersResourceWithRawResponse:
        return TopGainersLosersResourceWithRawResponse(self._coins.top_gainers_losers)

    @cached_property
    def circulating_supply_chart(self) -> CirculatingSupplyChartResourceWithRawResponse:
        return CirculatingSupplyChartResourceWithRawResponse(self._coins.circulating_supply_chart)

    @cached_property
    def contract(self) -> ContractResourceWithRawResponse:
        return ContractResourceWithRawResponse(self._coins.contract)

    @cached_property
    def history(self) -> HistoryResourceWithRawResponse:
        return HistoryResourceWithRawResponse(self._coins.history)

    @cached_property
    def market_chart(self) -> MarketChartResourceWithRawResponse:
        return MarketChartResourceWithRawResponse(self._coins.market_chart)

    @cached_property
    def ohlc(self) -> OhlcResourceWithRawResponse:
        return OhlcResourceWithRawResponse(self._coins.ohlc)

    @cached_property
    def tickers(self) -> TickersResourceWithRawResponse:
        return TickersResourceWithRawResponse(self._coins.tickers)

    @cached_property
    def total_supply_chart(self) -> TotalSupplyChartResourceWithRawResponse:
        return TotalSupplyChartResourceWithRawResponse(self._coins.total_supply_chart)


class AsyncCoinsResourceWithRawResponse:
    def __init__(self, coins: AsyncCoinsResource) -> None:
        self._coins = coins

        self.get_id = async_to_raw_response_wrapper(
            coins.get_id,
        )

    @cached_property
    def categories(self) -> AsyncCategoriesResourceWithRawResponse:
        return AsyncCategoriesResourceWithRawResponse(self._coins.categories)

    @cached_property
    def list(self) -> AsyncListResourceWithRawResponse:
        return AsyncListResourceWithRawResponse(self._coins.list)

    @cached_property
    def markets(self) -> AsyncMarketsResourceWithRawResponse:
        return AsyncMarketsResourceWithRawResponse(self._coins.markets)

    @cached_property
    def top_gainers_losers(self) -> AsyncTopGainersLosersResourceWithRawResponse:
        return AsyncTopGainersLosersResourceWithRawResponse(self._coins.top_gainers_losers)

    @cached_property
    def circulating_supply_chart(self) -> AsyncCirculatingSupplyChartResourceWithRawResponse:
        return AsyncCirculatingSupplyChartResourceWithRawResponse(self._coins.circulating_supply_chart)

    @cached_property
    def contract(self) -> AsyncContractResourceWithRawResponse:
        return AsyncContractResourceWithRawResponse(self._coins.contract)

    @cached_property
    def history(self) -> AsyncHistoryResourceWithRawResponse:
        return AsyncHistoryResourceWithRawResponse(self._coins.history)

    @cached_property
    def market_chart(self) -> AsyncMarketChartResourceWithRawResponse:
        return AsyncMarketChartResourceWithRawResponse(self._coins.market_chart)

    @cached_property
    def ohlc(self) -> AsyncOhlcResourceWithRawResponse:
        return AsyncOhlcResourceWithRawResponse(self._coins.ohlc)

    @cached_property
    def tickers(self) -> AsyncTickersResourceWithRawResponse:
        return AsyncTickersResourceWithRawResponse(self._coins.tickers)

    @cached_property
    def total_supply_chart(self) -> AsyncTotalSupplyChartResourceWithRawResponse:
        return AsyncTotalSupplyChartResourceWithRawResponse(self._coins.total_supply_chart)


class CoinsResourceWithStreamingResponse:
    def __init__(self, coins: CoinsResource) -> None:
        self._coins = coins

        self.get_id = to_streamed_response_wrapper(
            coins.get_id,
        )

    @cached_property
    def categories(self) -> CategoriesResourceWithStreamingResponse:
        return CategoriesResourceWithStreamingResponse(self._coins.categories)

    @cached_property
    def list(self) -> ListResourceWithStreamingResponse:
        return ListResourceWithStreamingResponse(self._coins.list)

    @cached_property
    def markets(self) -> MarketsResourceWithStreamingResponse:
        return MarketsResourceWithStreamingResponse(self._coins.markets)

    @cached_property
    def top_gainers_losers(self) -> TopGainersLosersResourceWithStreamingResponse:
        return TopGainersLosersResourceWithStreamingResponse(self._coins.top_gainers_losers)

    @cached_property
    def circulating_supply_chart(self) -> CirculatingSupplyChartResourceWithStreamingResponse:
        return CirculatingSupplyChartResourceWithStreamingResponse(self._coins.circulating_supply_chart)

    @cached_property
    def contract(self) -> ContractResourceWithStreamingResponse:
        return ContractResourceWithStreamingResponse(self._coins.contract)

    @cached_property
    def history(self) -> HistoryResourceWithStreamingResponse:
        return HistoryResourceWithStreamingResponse(self._coins.history)

    @cached_property
    def market_chart(self) -> MarketChartResourceWithStreamingResponse:
        return MarketChartResourceWithStreamingResponse(self._coins.market_chart)

    @cached_property
    def ohlc(self) -> OhlcResourceWithStreamingResponse:
        return OhlcResourceWithStreamingResponse(self._coins.ohlc)

    @cached_property
    def tickers(self) -> TickersResourceWithStreamingResponse:
        return TickersResourceWithStreamingResponse(self._coins.tickers)

    @cached_property
    def total_supply_chart(self) -> TotalSupplyChartResourceWithStreamingResponse:
        return TotalSupplyChartResourceWithStreamingResponse(self._coins.total_supply_chart)


class AsyncCoinsResourceWithStreamingResponse:
    def __init__(self, coins: AsyncCoinsResource) -> None:
        self._coins = coins

        self.get_id = async_to_streamed_response_wrapper(
            coins.get_id,
        )

    @cached_property
    def categories(self) -> AsyncCategoriesResourceWithStreamingResponse:
        return AsyncCategoriesResourceWithStreamingResponse(self._coins.categories)

    @cached_property
    def list(self) -> AsyncListResourceWithStreamingResponse:
        return AsyncListResourceWithStreamingResponse(self._coins.list)

    @cached_property
    def markets(self) -> AsyncMarketsResourceWithStreamingResponse:
        return AsyncMarketsResourceWithStreamingResponse(self._coins.markets)

    @cached_property
    def top_gainers_losers(self) -> AsyncTopGainersLosersResourceWithStreamingResponse:
        return AsyncTopGainersLosersResourceWithStreamingResponse(self._coins.top_gainers_losers)

    @cached_property
    def circulating_supply_chart(self) -> AsyncCirculatingSupplyChartResourceWithStreamingResponse:
        return AsyncCirculatingSupplyChartResourceWithStreamingResponse(self._coins.circulating_supply_chart)

    @cached_property
    def contract(self) -> AsyncContractResourceWithStreamingResponse:
        return AsyncContractResourceWithStreamingResponse(self._coins.contract)

    @cached_property
    def history(self) -> AsyncHistoryResourceWithStreamingResponse:
        return AsyncHistoryResourceWithStreamingResponse(self._coins.history)

    @cached_property
    def market_chart(self) -> AsyncMarketChartResourceWithStreamingResponse:
        return AsyncMarketChartResourceWithStreamingResponse(self._coins.market_chart)

    @cached_property
    def ohlc(self) -> AsyncOhlcResourceWithStreamingResponse:
        return AsyncOhlcResourceWithStreamingResponse(self._coins.ohlc)

    @cached_property
    def tickers(self) -> AsyncTickersResourceWithStreamingResponse:
        return AsyncTickersResourceWithStreamingResponse(self._coins.tickers)

    @cached_property
    def total_supply_chart(self) -> AsyncTotalSupplyChartResourceWithStreamingResponse:
        return AsyncTotalSupplyChartResourceWithStreamingResponse(self._coins.total_supply_chart)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .global_ import (
    GlobalResource,
    AsyncGlobalResource,
    GlobalResourceWithRawResponse,
    AsyncGlobalResourceWithRawResponse,
    GlobalResourceWithStreamingResponse,
    AsyncGlobalResourceWithStreamingResponse,
)
from .market_cap_chart import (
    MarketCapChartResource,
    AsyncMarketCapChartResource,
    MarketCapChartResourceWithRawResponse,
    AsyncMarketCapChartResourceWithRawResponse,
    MarketCapChartResourceWithStreamingResponse,
    AsyncMarketCapChartResourceWithStreamingResponse,
)
from .decentralized_finance_defi import (
    DecentralizedFinanceDefiResource,
    AsyncDecentralizedFinanceDefiResource,
    DecentralizedFinanceDefiResourceWithRawResponse,
    AsyncDecentralizedFinanceDefiResourceWithRawResponse,
    DecentralizedFinanceDefiResourceWithStreamingResponse,
    AsyncDecentralizedFinanceDefiResourceWithStreamingResponse,
)

__all__ = [
    "DecentralizedFinanceDefiResource",
    "AsyncDecentralizedFinanceDefiResource",
    "DecentralizedFinanceDefiResourceWithRawResponse",
    "AsyncDecentralizedFinanceDefiResourceWithRawResponse",
    "DecentralizedFinanceDefiResourceWithStreamingResponse",
    "AsyncDecentralizedFinanceDefiResourceWithStreamingResponse",
    "MarketCapChartResource",
    "AsyncMarketCapChartResource",
    "MarketCapChartResourceWithRawResponse",
    "AsyncMarketCapChartResourceWithRawResponse",
    "MarketCapChartResourceWithStreamingResponse",
    "AsyncMarketCapChartResourceWithStreamingResponse",
    "GlobalResource",
    "AsyncGlobalResource",
    "GlobalResourceWithRawResponse",
    "AsyncGlobalResourceWithRawResponse",
    "GlobalResourceWithStreamingResponse",
    "AsyncGlobalResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["MegafilterGetParams"]


class MegafilterGetParams(TypedDict, total=False):
    buy_tax_percentage_max: float
    """maximum buy tax percentage"""

    buy_tax_percentage_min: float
    """minimum buy tax percentage"""

    buys_duration: Literal["5m", "1h", "6h", "24h"]
    """duration for buy transactions metric Default value: 24h"""

    buys_max: int
    """maximum number of buy transactions"""

    buys_min: int
    """minimum number of buy transactions"""

    checks: str
    """
    filter options for various checks, comma-separated if more than one Available
    values: `no_honeypot`, `good_gt_score`, `on_coingecko`, `has_social`
    """

    dexes: str
    """
    filter pools by DEXes, comma-separated if more than one DEX ID refers to
    [/networks/{network}/dexes](/reference/dexes-list)
    """

    fdv_usd_max: float
    """maximum fully diluted value in USD"""

    fdv_usd_min: float
    """minimum fully diluted value in USD"""

    h24_volume_usd_max: float
    """maximum 24hr volume in USD"""

    h24_volume_usd_min: float
    """minimum 24hr volume in USD"""

    include: str
    """
    attributes to include, comma-separated if more than one to include Available
    values: `base_token`, `quote_token`, `dex`, `network`
    """

    networks: str
    """
    filter pools by networks, comma-separated if more than one Network ID refers to
    [/networks](/reference/networks-list)
    """

    page: int
    """page through results Default value: 1"""

    pool_created_hour_max: float
    """maximum pool age in hours"""

    pool_created_hour_min: float
    """minimum pool age in hours"""

    reserve_in_usd_max: float
    """maximum reserve in USD"""

    reserve_in_usd_min: float
    """minimum reserve in USD"""

    sell_tax_percentage_max: float
    """maximum sell tax percentage"""

    sell_tax_percentage_min: float
    """minimum sell tax percentage"""

    sells_duration: Literal["5m", "1h", "6h", "24h"]
    """duration for sell transactions metric Default value: 24h"""

    sells_max: int
    """maximum number of sell transactions"""

    sells_min: int
    """minimum number of sell transactions"""

    sort: Literal[
        "m5_trending",
        "h1_trending",
        "h6_trending",
        "h24_trending",
        "h24_tx_count_desc",
        "h24_volume_usd_desc",
        "h24_price_change_percentage_desc",
        "pool_created_at_desc",
    ]
    """sort the pools by field Default value: h6_trending"""

    tx_count_duration: Literal["5m", "1h", "6h", "24h"]
    """duration for transaction count metric Default value: 24h"""

    tx_count_max: int
    """maximum transaction count"""

    tx_count_min: int
    """minimum transaction count"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["TrendingSearchGetParams"]


class TrendingSearchGetParams(TypedDict, total=False):
    include: str
    """
    attributes to include, comma-separated if more than one to include Available
    values: `base_token`, `quote_token`, `dex`, `network`
    """

    pools: int
    """number of pools to return, maximum 10 Default value: 4"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.coins import ohlc_get_params, ohlc_get_range_params
from ..._base_client import make_request_options
from ...types.coins.ohlc_get_response import OhlcGetResponse
from ...types.coins.ohlc_get_range_response import OhlcGetRangeResponse

__all__ = ["OhlcResource", "AsyncOhlcResource"]


class OhlcResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> OhlcResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return OhlcResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OhlcResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return OhlcResourceWithStreamingResponse(self)

    def get(
        self,
        id: str,
        *,
        days: Literal["1", "7", "14", "30", "90", "180", "365", "max"],
        vs_currency: str,
        interval: Literal["daily", "hourly"] | NotGiven = NOT_GIVEN,
        precision: Literal[
            "full",
            "0",
            "1",
            "2",
            "3",
            "4",
            "5",
            "6",
            "7",
            "8",
            "9",
            "10",
            "11",
            "12",
            "13",
            "14",
            "15",
            "16",
            "17",
            "18",
        ]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OhlcGetResponse:
        """
        This endpoint allows you to **get the OHLC chart (Open, High, Low, Close) of a
        coin based on particular coin ID**

        Args:
          days: data up to number of days ago

          vs_currency: target currency of price data \\**refers to
              [`/simple/supported_vs_currencies`](/reference/simple-supported-currencies).

          interval: data interval, leave empty for auto granularity

          precision: decimal place for currency price value

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/coins/{id}/ohlc",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "days": days,
                        "vs_currency": vs_currency,
                        "interval": interval,
                        "precision": precision,
                    },
                    ohlc_get_params.OhlcGetParams,
                ),
            ),
            cast_to=OhlcGetResponse,
        )

    def get_range(
        self,
        id: str,
        *,
        from_: float,
        interval: Literal["daily", "hourly"],
        to: float,
        vs_currency: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OhlcGetRangeResponse:
        """
        This endpoint allows you to **get the OHLC chart (Open, High, Low, Close) of a
        coin within a range of timestamp based on particular coin ID**

        Args:
          from_: starting date in UNIX timestamp

          interval: data interval

          to: ending date in UNIX timestamp

          vs_currency: target currency of price data \\**refers to
              [`/simple/supported_vs_currencies`](/reference/simple-supported-currencies).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/coins/{id}/ohlc/range",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "from_": from_,
                        "interval": interval,
                        "to": to,
                        "vs_currency": vs_currency,
                    },
                    ohlc_get_range_params.OhlcGetRangeParams,
                ),
            ),
            cast_to=OhlcGetRangeResponse,
        )


class AsyncOhlcResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncOhlcResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOhlcResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOhlcResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncOhlcResourceWithStreamingResponse(self)

    async def get(
        self,
        id: str,
        *,
        days: Literal["1", "7", "14", "30", "90", "180", "365", "max"],
        vs_currency: str,
        interval: Literal["daily", "hourly"] | NotGiven = NOT_GIVEN,
        precision: Literal[
            "full",
            "0",
            "1",
            "2",
            "3",
            "4",
            "5",
            "6",
            "7",
            "8",
            "9",
            "10",
            "11",
            "12",
            "13",
            "14",
            "15",
            "16",
            "17",
            "18",
        ]
        | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OhlcGetResponse:
        """
        This endpoint allows you to **get the OHLC chart (Open, High, Low, Close) of a
        coin based on particular coin ID**

        Args:
          days: data up to number of days ago

          vs_currency: target currency of price data \\**refers to
              [`/simple/supported_vs_currencies`](/reference/simple-supported-currencies).

          interval: data interval, leave empty for auto granularity

          precision: decimal place for currency price value

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/coins/{id}/ohlc",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "days": days,
                        "vs_currency": vs_currency,
                        "interval": interval,
                        "precision": precision,
                    },
                    ohlc_get_params.OhlcGetParams,
                ),
            ),
            cast_to=OhlcGetResponse,
        )

    async def get_range(
        self,
        id: str,
        *,
        from_: float,
        interval: Literal["daily", "hourly"],
        to: float,
        vs_currency: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> OhlcGetRangeResponse:
        """
        This endpoint allows you to **get the OHLC chart (Open, High, Low, Close) of a
        coin within a range of timestamp based on particular coin ID**

        Args:
          from_: starting date in UNIX timestamp

          interval: data interval

          to: ending date in UNIX timestamp

          vs_currency: target currency of price data \\**refers to
              [`/simple/supported_vs_currencies`](/reference/simple-supported-currencies).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/coins/{id}/ohlc/range",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "from_": from_,
                        "interval": interval,
                        "to": to,
                        "vs_currency": vs_currency,
                    },
                    ohlc_get_range_params.OhlcGetRangeParams,
                ),
            ),
            cast_to=OhlcGetRangeResponse,
        )


class OhlcResourceWithRawResponse:
    def __init__(self, ohlc: OhlcResource) -> None:
        self._ohlc = ohlc

        self.get = to_raw_response_wrapper(
            ohlc.get,
        )
        self.get_range = to_raw_response_wrapper(
            ohlc.get_range,
        )


class AsyncOhlcResourceWithRawResponse:
    def __init__(self, ohlc: AsyncOhlcResource) -> None:
        self._ohlc = ohlc

        self.get = async_to_raw_response_wrapper(
            ohlc.get,
        )
        self.get_range = async_to_raw_response_wrapper(
            ohlc.get_range,
        )


class OhlcResourceWithStreamingResponse:
    def __init__(self, ohlc: OhlcResource) -> None:
        self._ohlc = ohlc

        self.get = to_streamed_response_wrapper(
            ohlc.get,
        )
        self.get_range = to_streamed_response_wrapper(
            ohlc.get_range,
        )


class AsyncOhlcResourceWithStreamingResponse:
    def __init__(self, ohlc: AsyncOhlcResource) -> None:
        self._ohlc = ohlc

        self.get = async_to_streamed_response_wrapper(
            ohlc.get,
        )
        self.get_range = async_to_streamed_response_wrapper(
            ohlc.get_range,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import TypeAlias

from .._models import BaseModel

__all__ = [
    "NFTGetMarketsResponse",
    "NFTGetMarketsResponseItem",
    "NFTGetMarketsResponseItemFloorPrice",
    "NFTGetMarketsResponseItemFloorPrice24hPercentageChange",
    "NFTGetMarketsResponseItemImage",
    "NFTGetMarketsResponseItemMarketCap",
    "NFTGetMarketsResponseItemMarketCap24hPercentageChange",
    "NFTGetMarketsResponseItemVolume24h",
    "NFTGetMarketsResponseItemVolume24hPercentageChange",
]


class NFTGetMarketsResponseItemFloorPrice(BaseModel):
    native_currency: Optional[float] = None

    usd: Optional[float] = None


class NFTGetMarketsResponseItemFloorPrice24hPercentageChange(BaseModel):
    native_currency: Optional[float] = None

    usd: Optional[float] = None


class NFTGetMarketsResponseItemImage(BaseModel):
    small: Optional[str] = None

    small_2x: Optional[str] = None


class NFTGetMarketsResponseItemMarketCap(BaseModel):
    native_currency: Optional[float] = None

    usd: Optional[float] = None


class NFTGetMarketsResponseItemMarketCap24hPercentageChange(BaseModel):
    native_currency: Optional[float] = None

    usd: Optional[float] = None


class NFTGetMarketsResponseItemVolume24h(BaseModel):
    native_currency: Optional[float] = None

    usd: Optional[float] = None


class NFTGetMarketsResponseItemVolume24hPercentageChange(BaseModel):
    native_currency: Optional[float] = None

    usd: Optional[float] = None


class NFTGetMarketsResponseItem(BaseModel):
    id: Optional[str] = None
    """NFT collection ID"""

    asset_platform_id: Optional[str] = None
    """NFT collection asset platform ID"""

    contract_address: Optional[str] = None
    """NFT collection contract address"""

    description: Optional[str] = None
    """NFT collection description"""

    floor_price: Optional[NFTGetMarketsResponseItemFloorPrice] = None
    """NFT collection floor price"""

    floor_price_24h_percentage_change: Optional[NFTGetMarketsResponseItemFloorPrice24hPercentageChange] = None
    """NFT collection floor price 24 hours percentage change"""

    floor_price_in_usd_24h_percentage_change: Optional[float] = None
    """NFT collection floor price in usd 24 hours percentage change"""

    image: Optional[NFTGetMarketsResponseItemImage] = None
    """NFT collection image url"""

    market_cap: Optional[NFTGetMarketsResponseItemMarketCap] = None
    """NFT collection market cap"""

    market_cap_24h_percentage_change: Optional[NFTGetMarketsResponseItemMarketCap24hPercentageChange] = None
    """NFT collection market cap 24 hours percentage change"""

    market_cap_rank: Optional[float] = None
    """coin market cap rank"""

    name: Optional[str] = None
    """NFT collection name"""

    native_currency: Optional[str] = None
    """NFT collection native currency"""

    native_currency_symbol: Optional[str] = None
    """NFT collection native currency symbol"""

    number_of_unique_addresses: Optional[float] = None
    """number of unique address owning the NFTs"""

    number_of_unique_addresses_24h_percentage_change: Optional[float] = None
    """number of unique address owning the NFTs 24 hours percentage change"""

    one_day_average_sale_price: Optional[float] = None
    """NFT collection one day average sale price"""

    one_day_average_sale_price_24h_percentage_change: Optional[float] = None
    """NFT collection one day average sale price 24 hours percentage change"""

    one_day_sales: Optional[float] = None
    """NFT collection one day sales"""

    one_day_sales_24h_percentage_change: Optional[float] = None
    """NFT collection one day sales 24 hours percentage change"""

    symbol: Optional[str] = None
    """NFT collection symbol"""

    total_supply: Optional[float] = None
    """NFT collection total supply"""

    volume_24h: Optional[NFTGetMarketsResponseItemVolume24h] = None
    """NFT collection volume in 24 hours"""

    volume_24h_percentage_change: Optional[NFTGetMarketsResponseItemVolume24hPercentageChange] = None
    """NFT collection volume in 24 hours percentage change"""

    volume_in_usd_24h_percentage_change: Optional[float] = None
    """NFT collection volume in usd 24 hours percentage change"""


NFTGetMarketsResponse: TypeAlias = List[NFTGetMarketsResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ....._models import BaseModel

__all__ = ["TopHolderGetResponse", "Data", "DataAttributes", "DataAttributesHolder"]


class DataAttributesHolder(BaseModel):
    address: Optional[str] = None

    amount: Optional[str] = None

    label: Optional[str] = None

    percentage: Optional[str] = None

    rank: Optional[float] = None

    value: Optional[str] = None


class DataAttributes(BaseModel):
    holders: Optional[List[DataAttributesHolder]] = None

    last_updated_at: Optional[str] = None


class Data(BaseModel):
    id: Optional[str] = None

    attributes: Optional[DataAttributes] = None

    type: Optional[str] = None


class TopHolderGetResponse(BaseModel):
    data: Optional[Data] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .trending_get_params import TrendingGetParams as TrendingGetParams
from .trending_get_response import TrendingGetResponse as TrendingGetResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from .info import (
    InfoResource,
    AsyncInfoResource,
    InfoResourceWithRawResponse,
    AsyncInfoResourceWithRawResponse,
    InfoResourceWithStreamingResponse,
    AsyncInfoResourceWithStreamingResponse,
)
from .multi import (
    MultiResource,
    AsyncMultiResource,
    MultiResourceWithRawResponse,
    AsyncMultiResourceWithRawResponse,
    MultiResourceWithStreamingResponse,
    AsyncMultiResourceWithStreamingResponse,
)
from .ohlcv import (
    OhlcvResource,
    AsyncOhlcvResource,
    OhlcvResourceWithRawResponse,
    AsyncOhlcvResourceWithRawResponse,
    OhlcvResourceWithStreamingResponse,
    AsyncOhlcvResourceWithStreamingResponse,
)
from .pools import (
    PoolsResource,
    AsyncPoolsResource,
    PoolsResourceWithRawResponse,
    AsyncPoolsResourceWithRawResponse,
    PoolsResourceWithStreamingResponse,
    AsyncPoolsResourceWithStreamingResponse,
)
from .trades import (
    TradesResource,
    AsyncTradesResource,
    TradesResourceWithRawResponse,
    AsyncTradesResourceWithRawResponse,
    TradesResourceWithStreamingResponse,
    AsyncTradesResourceWithStreamingResponse,
)
from ....._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ....._utils import maybe_transform, async_maybe_transform
from ....._compat import cached_property
from .top_holders import (
    TopHoldersResource,
    AsyncTopHoldersResource,
    TopHoldersResourceWithRawResponse,
    AsyncTopHoldersResourceWithRawResponse,
    TopHoldersResourceWithStreamingResponse,
    AsyncTopHoldersResourceWithStreamingResponse,
)
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .holders_chart import (
    HoldersChartResource,
    AsyncHoldersChartResource,
    HoldersChartResourceWithRawResponse,
    AsyncHoldersChartResourceWithRawResponse,
    HoldersChartResourceWithStreamingResponse,
    AsyncHoldersChartResourceWithStreamingResponse,
)
from ....._base_client import make_request_options
from .....types.onchain.networks import token_get_address_params
from .....types.onchain.networks.token_get_address_response import TokenGetAddressResponse

__all__ = ["TokensResource", "AsyncTokensResource"]


class TokensResource(SyncAPIResource):
    @cached_property
    def multi(self) -> MultiResource:
        return MultiResource(self._client)

    @cached_property
    def info(self) -> InfoResource:
        return InfoResource(self._client)

    @cached_property
    def top_holders(self) -> TopHoldersResource:
        return TopHoldersResource(self._client)

    @cached_property
    def holders_chart(self) -> HoldersChartResource:
        return HoldersChartResource(self._client)

    @cached_property
    def ohlcv(self) -> OhlcvResource:
        return OhlcvResource(self._client)

    @cached_property
    def pools(self) -> PoolsResource:
        return PoolsResource(self._client)

    @cached_property
    def trades(self) -> TradesResource:
        return TradesResource(self._client)

    @cached_property
    def with_raw_response(self) -> TokensResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return TokensResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TokensResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return TokensResourceWithStreamingResponse(self)

    def get_address(
        self,
        address: str,
        *,
        network: str,
        include: Literal["top_pools"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TokenGetAddressResponse:
        """
        This endpoint allows you to **query specific token data based on the provided
        token contract address on a network**

        Args:
          include: attributes to include

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not network:
            raise ValueError(f"Expected a non-empty value for `network` but received {network!r}")
        if not address:
            raise ValueError(f"Expected a non-empty value for `address` but received {address!r}")
        return self._get(
            f"/onchain/networks/{network}/tokens/{address}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"include": include}, token_get_address_params.TokenGetAddressParams),
            ),
            cast_to=TokenGetAddressResponse,
        )


class AsyncTokensResource(AsyncAPIResource):
    @cached_property
    def multi(self) -> AsyncMultiResource:
        return AsyncMultiResource(self._client)

    @cached_property
    def info(self) -> AsyncInfoResource:
        return AsyncInfoResource(self._client)

    @cached_property
    def top_holders(self) -> AsyncTopHoldersResource:
        return AsyncTopHoldersResource(self._client)

    @cached_property
    def holders_chart(self) -> AsyncHoldersChartResource:
        return AsyncHoldersChartResource(self._client)

    @cached_property
    def ohlcv(self) -> AsyncOhlcvResource:
        return AsyncOhlcvResource(self._client)

    @cached_property
    def pools(self) -> AsyncPoolsResource:
        return AsyncPoolsResource(self._client)

    @cached_property
    def trades(self) -> AsyncTradesResource:
        return AsyncTradesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncTokensResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncTokensResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTokensResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncTokensResourceWithStreamingResponse(self)

    async def get_address(
        self,
        address: str,
        *,
        network: str,
        include: Literal["top_pools"] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> TokenGetAddressResponse:
        """
        This endpoint allows you to **query specific token data based on the provided
        token contract address on a network**

        Args:
          include: attributes to include

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not network:
            raise ValueError(f"Expected a non-empty value for `network` but received {network!r}")
        if not address:
            raise ValueError(f"Expected a non-empty value for `address` but received {address!r}")
        return await self._get(
            f"/onchain/networks/{network}/tokens/{address}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"include": include}, token_get_address_params.TokenGetAddressParams),
            ),
            cast_to=TokenGetAddressResponse,
        )


class TokensResourceWithRawResponse:
    def __init__(self, tokens: TokensResource) -> None:
        self._tokens = tokens

        self.get_address = to_raw_response_wrapper(
            tokens.get_address,
        )

    @cached_property
    def multi(self) -> MultiResourceWithRawResponse:
        return MultiResourceWithRawResponse(self._tokens.multi)

    @cached_property
    def info(self) -> InfoResourceWithRawResponse:
        return InfoResourceWithRawResponse(self._tokens.info)

    @cached_property
    def top_holders(self) -> TopHoldersResourceWithRawResponse:
        return TopHoldersResourceWithRawResponse(self._tokens.top_holders)

    @cached_property
    def holders_chart(self) -> HoldersChartResourceWithRawResponse:
        return HoldersChartResourceWithRawResponse(self._tokens.holders_chart)

    @cached_property
    def ohlcv(self) -> OhlcvResourceWithRawResponse:
        return OhlcvResourceWithRawResponse(self._tokens.ohlcv)

    @cached_property
    def pools(self) -> PoolsResourceWithRawResponse:
        return PoolsResourceWithRawResponse(self._tokens.pools)

    @cached_property
    def trades(self) -> TradesResourceWithRawResponse:
        return TradesResourceWithRawResponse(self._tokens.trades)


class AsyncTokensResourceWithRawResponse:
    def __init__(self, tokens: AsyncTokensResource) -> None:
        self._tokens = tokens

        self.get_address = async_to_raw_response_wrapper(
            tokens.get_address,
        )

    @cached_property
    def multi(self) -> AsyncMultiResourceWithRawResponse:
        return AsyncMultiResourceWithRawResponse(self._tokens.multi)

    @cached_property
    def info(self) -> AsyncInfoResourceWithRawResponse:
        return AsyncInfoResourceWithRawResponse(self._tokens.info)

    @cached_property
    def top_holders(self) -> AsyncTopHoldersResourceWithRawResponse:
        return AsyncTopHoldersResourceWithRawResponse(self._tokens.top_holders)

    @cached_property
    def holders_chart(self) -> AsyncHoldersChartResourceWithRawResponse:
        return AsyncHoldersChartResourceWithRawResponse(self._tokens.holders_chart)

    @cached_property
    def ohlcv(self) -> AsyncOhlcvResourceWithRawResponse:
        return AsyncOhlcvResourceWithRawResponse(self._tokens.ohlcv)

    @cached_property
    def pools(self) -> AsyncPoolsResourceWithRawResponse:
        return AsyncPoolsResourceWithRawResponse(self._tokens.pools)

    @cached_property
    def trades(self) -> AsyncTradesResourceWithRawResponse:
        return AsyncTradesResourceWithRawResponse(self._tokens.trades)


class TokensResourceWithStreamingResponse:
    def __init__(self, tokens: TokensResource) -> None:
        self._tokens = tokens

        self.get_address = to_streamed_response_wrapper(
            tokens.get_address,
        )

    @cached_property
    def multi(self) -> MultiResourceWithStreamingResponse:
        return MultiResourceWithStreamingResponse(self._tokens.multi)

    @cached_property
    def info(self) -> InfoResourceWithStreamingResponse:
        return InfoResourceWithStreamingResponse(self._tokens.info)

    @cached_property
    def top_holders(self) -> TopHoldersResourceWithStreamingResponse:
        return TopHoldersResourceWithStreamingResponse(self._tokens.top_holders)

    @cached_property
    def holders_chart(self) -> HoldersChartResourceWithStreamingResponse:
        return HoldersChartResourceWithStreamingResponse(self._tokens.holders_chart)

    @cached_property
    def ohlcv(self) -> OhlcvResourceWithStreamingResponse:
        return OhlcvResourceWithStreamingResponse(self._tokens.ohlcv)

    @cached_property
    def pools(self) -> PoolsResourceWithStreamingResponse:
        return PoolsResourceWithStreamingResponse(self._tokens.pools)

    @cached_property
    def trades(self) -> TradesResourceWithStreamingResponse:
        return TradesResourceWithStreamingResponse(self._tokens.trades)


class AsyncTokensResourceWithStreamingResponse:
    def __init__(self, tokens: AsyncTokensResource) -> None:
        self._tokens = tokens

        self.get_address = async_to_streamed_response_wrapper(
            tokens.get_address,
        )

    @cached_property
    def multi(self) -> AsyncMultiResourceWithStreamingResponse:
        return AsyncMultiResourceWithStreamingResponse(self._tokens.multi)

    @cached_property
    def info(self) -> AsyncInfoResourceWithStreamingResponse:
        return AsyncInfoResourceWithStreamingResponse(self._tokens.info)

    @cached_property
    def top_holders(self) -> AsyncTopHoldersResourceWithStreamingResponse:
        return AsyncTopHoldersResourceWithStreamingResponse(self._tokens.top_holders)

    @cached_property
    def holders_chart(self) -> AsyncHoldersChartResourceWithStreamingResponse:
        return AsyncHoldersChartResourceWithStreamingResponse(self._tokens.holders_chart)

    @cached_property
    def ohlcv(self) -> AsyncOhlcvResourceWithStreamingResponse:
        return AsyncOhlcvResourceWithStreamingResponse(self._tokens.ohlcv)

    @cached_property
    def pools(self) -> AsyncPoolsResourceWithStreamingResponse:
        return AsyncPoolsResourceWithStreamingResponse(self._tokens.pools)

    @cached_property
    def trades(self) -> AsyncTradesResourceWithStreamingResponse:
        return AsyncTradesResourceWithStreamingResponse(self._tokens.trades)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["MultiGetAddressesParams"]


class MultiGetAddressesParams(TypedDict, total=False):
    network: Required[str]

    include: Literal["top_pools"]
    """attributes to include"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["TopGainersLoserGetParams"]


class TopGainersLoserGetParams(TypedDict, total=False):
    vs_currency: Required[str]
    """
    target currency of coins \\**refers to
    [`/simple/supported_vs_currencies`](/reference/simple-supported-currencies).
    """

    duration: Literal["1h", "24h", "7d", "14d", "30d", "60d", "1y"]
    """filter result by time range Default value: `24h`"""

    top_coins: Literal["300", "500", "1000", "all"]
    """
    filter result by market cap ranking (top 300 to 1000) or all coins (including
    coins that do not have market cap) Default value: `1000`
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["VolumeChartGetParams"]


class VolumeChartGetParams(TypedDict, total=False):
    days: Required[Literal["1", "7", "14", "30", "90", "180", "365"]]
    """data up to number of days ago"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["SearchGetResponse", "Category", "Coin", "Exchange", "NFT"]


class Category(BaseModel):
    id: Optional[str] = None
    """category ID"""

    name: Optional[str] = None
    """category name"""


class Coin(BaseModel):
    id: Optional[str] = None
    """coin ID"""

    api_symbol: Optional[str] = None
    """coin api symbol"""

    large: Optional[str] = None
    """coin large image url"""

    market_cap_rank: Optional[float] = None
    """coin market cap rank"""

    name: Optional[str] = None
    """coin name"""

    symbol: Optional[str] = None
    """coin symbol"""

    thumb: Optional[str] = None
    """coin thumb image url"""


class Exchange(BaseModel):
    id: Optional[str] = None
    """exchange ID"""

    large: Optional[str] = None
    """exchange large image url"""

    market_type: Optional[str] = None
    """exchange market type"""

    name: Optional[str] = None
    """exchange name"""

    thumb: Optional[str] = None
    """exchange thumb image url"""


class NFT(BaseModel):
    id: Optional[str] = None
    """NFT collection ID"""

    name: Optional[str] = None
    """NFT name"""

    symbol: Optional[str] = None
    """NFT collection symbol"""

    thumb: Optional[str] = None
    """NFT collection thumb image url"""


class SearchGetResponse(BaseModel):
    categories: Optional[List[Category]] = None

    coins: Optional[List[Coin]] = None

    exchanges: Optional[List[Exchange]] = None

    icos: Optional[List[str]] = None

    nfts: Optional[List[NFT]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .nfts import (
    NFTsResource,
    AsyncNFTsResource,
    NFTsResourceWithRawResponse,
    AsyncNFTsResourceWithRawResponse,
    NFTsResourceWithStreamingResponse,
    AsyncNFTsResourceWithStreamingResponse,
)
from .tickers import (
    TickersResource,
    AsyncTickersResource,
    TickersResourceWithRawResponse,
    AsyncTickersResourceWithRawResponse,
    TickersResourceWithStreamingResponse,
    AsyncTickersResourceWithStreamingResponse,
)
from .contract import (
    ContractResource,
    AsyncContractResource,
    ContractResourceWithRawResponse,
    AsyncContractResourceWithRawResponse,
    ContractResourceWithStreamingResponse,
    AsyncContractResourceWithStreamingResponse,
)
from .market_chart import (
    MarketChartResource,
    AsyncMarketChartResource,
    MarketChartResourceWithRawResponse,
    AsyncMarketChartResourceWithRawResponse,
    MarketChartResourceWithStreamingResponse,
    AsyncMarketChartResourceWithStreamingResponse,
)

__all__ = [
    "ContractResource",
    "AsyncContractResource",
    "ContractResourceWithRawResponse",
    "AsyncContractResourceWithRawResponse",
    "ContractResourceWithStreamingResponse",
    "AsyncContractResourceWithStreamingResponse",
    "MarketChartResource",
    "AsyncMarketChartResource",
    "MarketChartResourceWithRawResponse",
    "AsyncMarketChartResourceWithRawResponse",
    "MarketChartResourceWithStreamingResponse",
    "AsyncMarketChartResourceWithStreamingResponse",
    "TickersResource",
    "AsyncTickersResource",
    "TickersResourceWithRawResponse",
    "AsyncTickersResourceWithRawResponse",
    "TickersResourceWithStreamingResponse",
    "AsyncTickersResourceWithStreamingResponse",
    "NFTsResource",
    "AsyncNFTsResource",
    "NFTsResourceWithRawResponse",
    "AsyncNFTsResourceWithRawResponse",
    "NFTsResourceWithStreamingResponse",
    "AsyncNFTsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .price import (
    PriceResource,
    AsyncPriceResource,
    PriceResourceWithRawResponse,
    AsyncPriceResourceWithRawResponse,
    PriceResourceWithStreamingResponse,
    AsyncPriceResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .token_price import (
    TokenPriceResource,
    AsyncTokenPriceResource,
    TokenPriceResourceWithRawResponse,
    AsyncTokenPriceResourceWithRawResponse,
    TokenPriceResourceWithStreamingResponse,
    AsyncTokenPriceResourceWithStreamingResponse,
)
from .supported_vs_currencies import (
    SupportedVsCurrenciesResource,
    AsyncSupportedVsCurrenciesResource,
    SupportedVsCurrenciesResourceWithRawResponse,
    AsyncSupportedVsCurrenciesResourceWithRawResponse,
    SupportedVsCurrenciesResourceWithStreamingResponse,
    AsyncSupportedVsCurrenciesResourceWithStreamingResponse,
)

__all__ = ["SimpleResource", "AsyncSimpleResource"]


class SimpleResource(SyncAPIResource):
    @cached_property
    def price(self) -> PriceResource:
        return PriceResource(self._client)

    @cached_property
    def supported_vs_currencies(self) -> SupportedVsCurrenciesResource:
        return SupportedVsCurrenciesResource(self._client)

    @cached_property
    def token_price(self) -> TokenPriceResource:
        return TokenPriceResource(self._client)

    @cached_property
    def with_raw_response(self) -> SimpleResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return SimpleResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SimpleResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return SimpleResourceWithStreamingResponse(self)


class AsyncSimpleResource(AsyncAPIResource):
    @cached_property
    def price(self) -> AsyncPriceResource:
        return AsyncPriceResource(self._client)

    @cached_property
    def supported_vs_currencies(self) -> AsyncSupportedVsCurrenciesResource:
        return AsyncSupportedVsCurrenciesResource(self._client)

    @cached_property
    def token_price(self) -> AsyncTokenPriceResource:
        return AsyncTokenPriceResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSimpleResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSimpleResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSimpleResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncSimpleResourceWithStreamingResponse(self)


class SimpleResourceWithRawResponse:
    def __init__(self, simple: SimpleResource) -> None:
        self._simple = simple

    @cached_property
    def price(self) -> PriceResourceWithRawResponse:
        return PriceResourceWithRawResponse(self._simple.price)

    @cached_property
    def supported_vs_currencies(self) -> SupportedVsCurrenciesResourceWithRawResponse:
        return SupportedVsCurrenciesResourceWithRawResponse(self._simple.supported_vs_currencies)

    @cached_property
    def token_price(self) -> TokenPriceResourceWithRawResponse:
        return TokenPriceResourceWithRawResponse(self._simple.token_price)


class AsyncSimpleResourceWithRawResponse:
    def __init__(self, simple: AsyncSimpleResource) -> None:
        self._simple = simple

    @cached_property
    def price(self) -> AsyncPriceResourceWithRawResponse:
        return AsyncPriceResourceWithRawResponse(self._simple.price)

    @cached_property
    def supported_vs_currencies(self) -> AsyncSupportedVsCurrenciesResourceWithRawResponse:
        return AsyncSupportedVsCurrenciesResourceWithRawResponse(self._simple.supported_vs_currencies)

    @cached_property
    def token_price(self) -> AsyncTokenPriceResourceWithRawResponse:
        return AsyncTokenPriceResourceWithRawResponse(self._simple.token_price)


class SimpleResourceWithStreamingResponse:
    def __init__(self, simple: SimpleResource) -> None:
        self._simple = simple

    @cached_property
    def price(self) -> PriceResourceWithStreamingResponse:
        return PriceResourceWithStreamingResponse(self._simple.price)

    @cached_property
    def supported_vs_currencies(self) -> SupportedVsCurrenciesResourceWithStreamingResponse:
        return SupportedVsCurrenciesResourceWithStreamingResponse(self._simple.supported_vs_currencies)

    @cached_property
    def token_price(self) -> TokenPriceResourceWithStreamingResponse:
        return TokenPriceResourceWithStreamingResponse(self._simple.token_price)


class AsyncSimpleResourceWithStreamingResponse:
    def __init__(self, simple: AsyncSimpleResource) -> None:
        self._simple = simple

    @cached_property
    def price(self) -> AsyncPriceResourceWithStreamingResponse:
        return AsyncPriceResourceWithStreamingResponse(self._simple.price)

    @cached_property
    def supported_vs_currencies(self) -> AsyncSupportedVsCurrenciesResourceWithStreamingResponse:
        return AsyncSupportedVsCurrenciesResourceWithStreamingResponse(self._simple.supported_vs_currencies)

    @cached_property
    def token_price(self) -> AsyncTokenPriceResourceWithStreamingResponse:
        return AsyncTokenPriceResourceWithStreamingResponse(self._simple.token_price)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from .networks.networks import (
    NetworksResource,
    AsyncNetworksResource,
    NetworksResourceWithRawResponse,
    AsyncNetworksResourceWithRawResponse,
    NetworksResourceWithStreamingResponse,
    AsyncNetworksResourceWithStreamingResponse,
)

__all__ = ["SimpleResource", "AsyncSimpleResource"]


class SimpleResource(SyncAPIResource):
    @cached_property
    def networks(self) -> NetworksResource:
        return NetworksResource(self._client)

    @cached_property
    def with_raw_response(self) -> SimpleResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return SimpleResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SimpleResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return SimpleResourceWithStreamingResponse(self)


class AsyncSimpleResource(AsyncAPIResource):
    @cached_property
    def networks(self) -> AsyncNetworksResource:
        return AsyncNetworksResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSimpleResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/coingecko/coingecko-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSimpleResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSimpleResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/coingecko/coingecko-python#with_streaming_response
        """
        return AsyncSimpleResourceWithStreamingResponse(self)


class SimpleResourceWithRawResponse:
    def __init__(self, simple: SimpleResource) -> None:
        self._simple = simple

    @cached_property
    def networks(self) -> NetworksResourceWithRawResponse:
        return NetworksResourceWithRawResponse(self._simple.networks)


class AsyncSimpleResourceWithRawResponse:
    def __init__(self, simple: AsyncSimpleResource) -> None:
        self._simple = simple

    @cached_property
    def networks(self) -> AsyncNetworksResourceWithRawResponse:
        return AsyncNetworksResourceWithRawResponse(self._simple.networks)


class SimpleResourceWithStreamingResponse:
    def __init__(self, simple: SimpleResource) -> None:
        self._simple = simple

    @cached_property
    def networks(self) -> NetworksResourceWithStreamingResponse:
        return NetworksResourceWithStreamingResponse(self._simple.networks)


class AsyncSimpleResourceWithStreamingResponse:
    def __init__(self, simple: AsyncSimpleResource) -> None:
        self._simple = simple

    @cached_property
    def networks(self) -> AsyncNetworksResourceWithStreamingResponse:
        return AsyncNetworksResourceWithStreamingResponse(self._simple.networks)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["MarketChartGetParams"]


class MarketChartGetParams(TypedDict, total=False):
    asset_platform_id: Required[str]

    days: Required[str]
    """data up to number of days ago Valid values: any integer or max"""
